/*********************************
Patch for CardioLog 2011 [2.0.3.1]
**********************************/

Use CardioLog
GO

-- BUG: The Average Visit Duration and Average Number of Unique Users reports display wrong decimal units in report generated mode.
ALTER procedure [dbo].[stp_eventlog_visits_avg_duration_chart]
  @startTime datetime,
  @endTime datetime,
  @groupIds varchar(2000),
  @userIds varchar(2000),
  @categoryIds varchar(2000),
  @eventType smallint,
  @spLocation varchar(500),
  @aggregated tinyint,
  @timeInterval varchar(20)
AS

declare @sql varchar(8000)
set @timeInterval = dbo.func_eventlog_timeinterval(@timeInterval)
									
set @sql = '
select startTime, convert(int, round(avg(_sum),0)) [count]
from
(
select ' + dbo.func_eventlog_datetime_floor(@timeInterval) + ' startTime, sum(cast(cast(param1 as bigint) as decimal)/1000) _sum
from
	(
	select C.*
	from
		tab_event_log C'

set @sql = @sql + dbo.func_eventlog_filters(@groupIds,@userIds,@categoryIds,@startTime,@endTime,@eventType,@spLocation,'',@aggregated);
set @sql = @sql + '
	)T'		
set @sql = @sql + '
group by ' + dbo.func_eventlog_datetime_floor(@timeInterval) + ', T.sessionid
) T
group by startTime
'

exec(@sql)

return @@error  -- 0 = ok, other = error

GO

-- BUG: Object Explorer search and item properties do not return results when the total size of all SharePoint tree items > 4 GB.

insert into tab_global_settings
values ('CardioLog', 'TopXSearchResults', '50')

GO

ALTER Procedure [dbo].[stp_sharepoint_tree_pathbylocations]
	@locations varchar(2000)
AS

	declare @sql as varchar(4000);
	
	set @sql = 'SELECT id, url, entityType, location, name, title, ParentId, Template, Owner, Created, Modified, Version, IsDeleted, DeletionDate, NextChildId, isManual, ReferenceId, SPID, Size, Editor, Extension FROM TAB_SHAREPOINT_TREE WHERE [location] in (' + @locations + ') ORDER BY [Location] ASC';
	
	exec (@sql);

	return @@error  -- 0 = ok, other = error
	
GO

ALTER Procedure [dbo].[stp_sharepoint_tree_search]
	@exact bit,
	@field varchar(200),
	@value nvarchar(2000),
	@includeDeleted bit,
	@includeListItems bit
AS
	
	declare @sql as nvarchar(4000);
	declare @topX as nvarchar(100);
	set @topX = (select value from tab_global_settings t
					where t.Category = 'CardioLog' and t.Name = 'TopXSearchResults')

	set @sql = 'SELECT Top ' + @topX + ' id, url, entityType, location, name, title, ParentId, Template, Owner, Created, Modified, Version, IsDeleted, DeletionDate, NextChildId, isManual, ReferenceId, SPID, Size, Editor, Extension FROM	TAB_SHAREPOINT_TREE WHERE ' + @field;
	
	IF (@exact = 1 )
	BEGIN
		set @sql = @sql + ' = N''' + @value + ''' ';
	END
	ELSE
	BEGIN
		set @sql = @sql + ' like N''%' + @value + '%'' ';
	END
	
	IF (@includeDeleted = 0) 
		set @sql = @sql + ' AND isDeleted = 0 ';

	IF (@includeListItems = 0) 
		set @sql = @sql + ' AND EntityType <> 5 ';
	
	exec(@sql);

	return @@error  -- 0 = ok, other = error	
	
GO

-- CHG: Improved performance for the Usage Data Processing service in case of over 100,000 temp events.

ALTER procedure [dbo].[stp_eventlog_migrate]
	@blackListFilter nvarchar(max)
as
	INSERT INTO tab_log values(getdate(),'MIGRATE','Start');
	
	declare @maxEventLogId int;
	declare @minEventLogId int;
	declare @blackListFilterSQL_1 nvarchar(max);
	declare @blackListFilterSQL_2 nvarchar(max);
	declare @TransactionRowCount int;
	declare @TransactionSize int;
	declare @error int;

	declare @urlCollationMatch int
	declare @queryStringCollationMatch int
	
	select @TransactionSize=10000
	
	-- check collation match
	select	@urlCollationMatch = case when t1.collation_name = t2.collation_name then 1 else 0 end
	from	(select collation_name from sys.columns where object_id = object_id('TAB_SHAREPOINT_TREE') and name = 'url') t1
	cross join (select collation_name from sys.columns where object_id = object_id('TAB_TEMP_EVENT_LOG') and name = 'url') t2
	
	select	@queryStringCollationMatch = case when t1.collation_name = t2.collation_name then 1 else 0 end
	from	(select collation_name from sys.columns where object_id = object_id('TAB_SHAREPOINT_TREE') and name = 'url') t1
	cross join (select collation_name from sys.columns where object_id = object_id('TAB_TEMP_EVENT_LOG') and name = 'QueryString') t2

	select @maxEventLogId = max(eventlogid) from tab_temp_event_log;
	select @minEventLogId = min(eventlogid) from tab_temp_event_log;
		
	set @blackListFilterSQL_1 = '';
	set @blackListFilterSQL_2 = '';
	
	if ( len( @blackListFilter ) > 0 )
	begin
		set @blackListFilterSQL_1 = '
		insert into tab_event_log  ([EventLogId]
           ,[EntityId]
           ,[EntityType]
           ,[EventType]
           ,[Url]
           ,[UserName]
           ,[SessionId]
           ,[Timestamp]
           ,[Param1]
           ,[Param2]
           ,[Param3]
           ,[QueryString]
           ,[Created]
           ,[FromEventId]
           ,[SPLocation]
           ,[flyweightId]
           ,[ToEventId]
           ,[CategoryFlyweightId]
           ,[browserType]
           ,[clientIP]
		   ,[SearchURL]
		   ,[WorkStationIP]
           )' +
			  'select [EventLogId]
           ,[EntityId]
           ,[EntityType]
           ,[EventType]
           ,[Url]
           ,[UserName]
           ,[SessionId]
           ,[Timestamp]
           ,[Param1]
           ,[Param2]
           ,[Param3]
           ,[QueryString]
           ,[Created]
           ,[FromEventId]
           ,[SPLocation]
           ,[flyweightId]
           ,[ToEventId]
           ,[CategoryFlyweightId]
           ,[browserType]
           ,[clientIP]
		   ,[SearchURL]
		   ,[WorkStationIP]
            ' +
			  'from tab_migrate_event_log ' +
			  'where eventlogid not in ( ' +
					 'select eventlogid ' +
					 'from tab_migrate_event_log ' +
					 'where ' + @blacklistfilter + ' );
		'
		
		set @blackListFilterSQL_2 = '
		insert into tab_event_log_users (eventlogid,flyweightId,CategoryFlyweightId)
			select EventLogId, flyweightId, CategoryFlyweightId ' +
			  'from tab_migrate_event_log ' +
			  'where eventlogid not in ( ' +
					 'select eventlogid ' +
					 'from tab_migrate_event_log ' +
					 'where ' + @blacklistfilter + ' );
		'
	end	
	else
		set @blackListFilterSQL_1 = '
		insert into tab_event_log ([EventLogId]
           ,[EntityId]
           ,[EntityType]
           ,[EventType]
           ,[Url]
           ,[UserName]
           ,[SessionId]
           ,[Timestamp]
           ,[Param1]
           ,[Param2]
           ,[Param3]
           ,[QueryString]
           ,[Created]
           ,[FromEventId]
           ,[SPLocation]
           ,[flyweightId]
           ,[ToEventId]
           ,[CategoryFlyweightId]
           ,[browserType]
           ,[clientIP]
		   ,[SearchURL]
		   ,[WorkStationIP]
           )' +
			  'select [EventLogId]
           ,[EntityId]
           ,[EntityType]
           ,[EventType]
           ,[Url]
           ,[UserName]
           ,[SessionId]
           ,[Timestamp]
           ,[Param1]
           ,[Param2]
           ,[Param3]
           ,[QueryString]
           ,[Created]
           ,[FromEventId]
           ,[SPLocation]
           ,[flyweightId]
           ,[ToEventId]
           ,[CategoryFlyweightId]
           ,[browserType]
           ,[clientIP]
			,[SearchURL]
			,[WorkStationIP]
           ' +
			  'from tab_migrate_event_log

		insert into tab_event_log_users (eventlogid,flyweightId,CategoryFlyweightId)
			select EventLogId, flyweightId, CategoryFlyweightId
			  from tab_migrate_event_log 
		'

begin try

print 'step #1'
select @TransactionRowCount=1

WHILE @TransactionRowCount > 0
BEGIN
	begin tran [MainTrn]
	select @TransactionRowCount=0
	
	if	@queryStringCollationMatch = 0 
	begin
		INSERT 
        INTO        tab_migrate_event_log 
        SELECT top (@TransactionSize) 
              eventLog.EventLogId,
              tree.[Id],
              tree.EntityType,
              eventLog.EventType,
              SUBSTRING(eventLog.QueryString,1,1000),
              eventLog.UserName, 
              eventLog.SessionId,
              eventLog.[TimeStamp],
              eventLog.Param1,
              eventLog.Param2,
              eventLog.Param3,
              eventLog.QueryString,
              eventLog.Created,
              eventLog.FromEventId,
              tree.Location,
              eventLog.flyweightId,
              eventLog.ToEventId,
              eventLog.CategoryFlyweightId,
              eventLog.browserType,
              eventLog.clientIP,
			  substring(eventLog.QueryString,1,400) as [SearchURL],
			  eventLog.WorkStationIP
        FROM        
              TAB_TEMP_EVENT_LOG eventLog, 
              TAB_SHAREPOINT_TREE tree
        WHERE       
              eventLog.EventLogId  <= @maxEventLogId AND
              SUBSTRING(eventLog.QueryString,1,1000) COLLATE DATABASE_DEFAULT = tree.Url COLLATE DATABASE_DEFAULT AND
              tree.isDeleted = 0;
	end
	else
	begin
		INSERT 
        INTO        tab_migrate_event_log 
        SELECT top (@TransactionSize) 
              eventLog.EventLogId,
              tree.[Id],
              tree.EntityType,
              eventLog.EventType,
              SUBSTRING(eventLog.QueryString,1,1000),
              eventLog.UserName, 
              eventLog.SessionId,
              eventLog.[TimeStamp],
              eventLog.Param1,
              eventLog.Param2,
              eventLog.Param3,
              eventLog.QueryString,
              eventLog.Created,
              eventLog.FromEventId,
              tree.Location,
              eventLog.flyweightId,
              eventLog.ToEventId,
              eventLog.CategoryFlyweightId,
              eventLog.browserType,
              eventLog.clientIP,
			  substring(eventLog.QueryString,1,400) as [SearchURL],
			  eventLog.WorkStationIP
        FROM        
              TAB_TEMP_EVENT_LOG eventLog, 
              TAB_SHAREPOINT_TREE tree
        WHERE       
              eventLog.EventLogId  <= @maxEventLogId AND
              SUBSTRING(eventLog.QueryString,1,1000) = tree.Url AND
              tree.isDeleted = 0;	
	end	
		
		--------------------------------------------------
		-- Remove duplicated from tab_migrate_event_log
		--------------------------------------------------
		
		select @TransactionRowCount = @TransactionRowCount+@@rowcount
		exec [stp_eventlog_migrate_remove_dup]

        -- Delete from tab_migrate_event_log the black list events
        exec (@blackListFilterSQL_1 + @blackListFilterSQL_2)

     
print 'step #2'
		
      	DELETE 
        FROM TAB_TEMP_EVENT_LOG 
        WHERE       EXISTS (SELECT    EventLogId 
                                FROM TAB_MIGRATE_EVENT_LOG
                                WHERE TAB_MIGRATE_EVENT_LOG.EventLogId = TAB_TEMP_EVENT_LOG.EventLogId);
		

				
		TRUNCATE TABLE TAB_MIGRATE_EVENT_LOG;
	commit tran [MainTrn]		
END --while

print 'step #3'
select @TransactionRowCount=1

WHILE @TransactionRowCount > 0
BEGIN
	begin tran [MainTrn]
	select @TransactionRowCount=0
	if	@urlCollationMatch = 0 
	begin
		INSERT 
        INTO        tab_migrate_event_log 
        SELECT top (@TransactionSize)
              eventLog.EventLogId,
              tree.[Id],
              tree.EntityType,
              eventLog.EventType,
              eventLog.Url,
              eventLog.UserName, 
              eventLog.SessionId,
              eventLog.[TimeStamp],
              eventLog.Param1,
              eventLog.Param2,
              eventLog.Param3,
              eventLog.QueryString,
              eventLog.Created,
              eventLog.FromEventId,
              tree.Location,
              eventLog.flyweightId,
              eventLog.ToEventId,
              eventLog.CategoryFlyweightId,
              eventLog.browserType,
              eventLog.clientIP,
			  substring(eventLog.Url,1,400) as [SearchURL],
			  eventLog.WorkStationIP
        FROM        
              TAB_TEMP_EVENT_LOG eventLog, 
              TAB_SHAREPOINT_TREE tree
        WHERE       
              eventLog.EventLogId <= @maxEventLogId AND
              eventLog.Url COLLATE DATABASE_DEFAULT = tree.Url COLLATE DATABASE_DEFAULT AND
              SUBSTRING(eventLog.QueryString,1,1000)  <> eventLog.Url AND
              tree.isDeleted = 0;

	end
	else
	begin
		INSERT 
        INTO        tab_migrate_event_log 
        SELECT top (@TransactionSize)
              eventLog.EventLogId,
              tree.[Id],
              tree.EntityType,
              eventLog.EventType,
              eventLog.Url,
              eventLog.UserName, 
              eventLog.SessionId,
              eventLog.[TimeStamp],
              eventLog.Param1,
              eventLog.Param2,
              eventLog.Param3,
              eventLog.QueryString,
              eventLog.Created,
              eventLog.FromEventId,
              tree.Location,
              eventLog.flyweightId,
              eventLog.ToEventId,
              eventLog.CategoryFlyweightId,
              eventLog.browserType,
              eventLog.clientIP,
			  substring(eventLog.Url,1,400) as [SearchURL],
			  eventLog.WorkStationIP
        FROM        
              TAB_TEMP_EVENT_LOG eventLog, 
              TAB_SHAREPOINT_TREE tree
        WHERE       
              eventLog.EventLogId <= @maxEventLogId AND
              eventLog.Url = tree.Url AND
              SUBSTRING(eventLog.QueryString,1,1000)  <> eventLog.Url AND
              tree.isDeleted = 0;
	end
		--------------------------------------------------
		-- Remove duplicated from tab_migrate_event_log
		--------------------------------------------------
		
		select @TransactionRowCount = @TransactionRowCount+@@rowcount
		exec [stp_eventlog_migrate_remove_dup]

        -- Delete from tab_migrate_event_log the black list events
        exec (@blackListFilterSQL_1 + @blackListFilterSQL_2)

print 'step #4'
 
        DELETE 
        FROM TAB_TEMP_EVENT_LOG 
        WHERE       EXISTS (SELECT    EventLogId 
                                FROM TAB_MIGRATE_EVENT_LOG
                                WHERE TAB_MIGRATE_EVENT_LOG.EventLogId = TAB_TEMP_EVENT_LOG.EventLogId);

		TRUNCATE TABLE TAB_MIGRATE_EVENT_LOG;
	commit tran [MainTrn]	
END --while
 
     
print 'step #5'
select @TransactionRowCount=1

WHILE @TransactionRowCount > 0
BEGIN
	begin tran [MainTrn]      
	select @TransactionRowCount=0
    INSERT 
    INTO        tab_migrate_event_log 
    SELECT top (@TransactionSize)
          eventLog.EventLogId,
          eventLog.EntityId,
          eventLog.EntityType,
          eventLog.EventType,
          eventLog.Url,
          eventLog.UserName, 
          eventLog.SessionId,
          eventLog.[TimeStamp],
          eventLog.Param1,
          eventLog.Param2,
          eventLog.Param3,
          eventLog.QueryString,
          eventLog.Created,
          eventLog.FromEventId,
          eventLog.SPLocation,
          eventLog.flyweightId,
          eventLog.ToEventId,
          eventLog.CategoryFlyweightId,
          eventlog.browserType,
          eventlog.clientIP,
		  substring(eventLog.Url,1,400) as [SearchURL],
		  eventLog.WorkStationIP
    FROM        
          TAB_TEMP_EVENT_LOG eventLog
    WHERE       
                eventLog.EventLogId <= @maxEventLogId
 --         AND   (
				--	eventLog.[TimeStamp] < GetDate() - 3
				--	or
				--	eventLog.EntityType = -2
				--	or
				--	eventLog.EventType in (9,10)
				--)
		

	--------------------------------------------------
	-- Remove duplicated from tab_migrate_event_log
	--------------------------------------------------
	
	select @TransactionRowCount = @TransactionRowCount+@@rowcount
	exec [stp_eventlog_migrate_remove_dup]
	
    -- Delete from tab_migrate_event_log the black list events
	exec (@blackListFilterSQL_1 + @blackListFilterSQL_2)
			
print 'step #6'
 
        DELETE 
        FROM TAB_TEMP_EVENT_LOG 
        WHERE       EXISTS (SELECT    EventLogId 
                                FROM TAB_MIGRATE_EVENT_LOG
                                WHERE TAB_MIGRATE_EVENT_LOG.EventLogId = TAB_TEMP_EVENT_LOG.EventLogId);
 
		TRUNCATE TABLE TAB_MIGRATE_EVENT_LOG;
	commit tran [MainTrn]
END --while

print 'finish'


--INSERT INTO tab_log values(getdate(),'MIGRATE','Transaction Commited');

print @TransactionRowCount
--END -- End While
end try

begin catch

	DECLARE 
        @ErrorMessage    NVARCHAR(4000),
        @ErrorNumber     INT,
        @ErrorSeverity   INT,
        @ErrorState      INT,
        @ErrorLine       INT,
        @ErrorProcedure  NVARCHAR(200);

	SELECT 
        @ErrorNumber = ERROR_NUMBER(),
        @ErrorSeverity = ERROR_SEVERITY(),
        @ErrorState = ERROR_STATE(),
        @ErrorLine = ERROR_LINE(),
        @ErrorProcedure = ISNULL(ERROR_PROCEDURE(), '-');

		SELECT @ErrorMessage = 
        N'Error %d, Level %d, State %d, Procedure %s, Line %d, ' + 
            'Message: '+ ERROR_MESSAGE();


	IF (XACT_STATE()) <> 0
    BEGIN
        ROLLBACK TRANSACTION;
    END

	RAISERROR 
        (
        @ErrorMessage, 
        @ErrorSeverity, 
        1,               
        @ErrorNumber,
        @ErrorSeverity,
        @ErrorState,
        @ErrorProcedure,
        @ErrorLine
        );

end catch


declare @startTime datetime;
set @startTime = (select timestamp from tab_event_log where EventLogId = @minEventLogId)

--fix lost events folders
begin try
	INSERT INTO tab_log values(getdate(),'MIGRATE','START - fix lost events folders. @startTime: ' + convert(varchar(23),@startTime,21));
	exec stp_eventlog_fix_lost_events_folders @startTime
	INSERT INTO tab_log values(getdate(),'MIGRATE','START - end lost events folders');
end try
begin catch
end catch

--referrer log migrate
begin try
	INSERT INTO tab_log values(getdate(),'MIGRATE','START - referrer log set locations. @minEventLogId: ' + cast(@minEventLogId as varchar(20)));
	exec stp_referrerlog_set_locations @minEventLogId
	INSERT INTO tab_log values(getdate(),'MIGRATE','END - referrer log set locations');
end try
begin catch
end catch

GO

-- Update CardioLog patch version
update tab_global_settings
set Value = '2.0.3.1'
where Category = 'CardioLog' and Name = 'Version'