using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using CardioLog.API;
using System.Text;
using System.Text.RegularExpressions;
using System.Configuration;

namespace CardioLog.Tree
{
	public partial class TreeData : System.Web.UI.Page
	{
		private string action = string.Empty;
		private string username = string.Empty;
		private string connectedUsername = string.Empty;
		private NavigatorType treeType;
		private string entityType = string.Empty;

		public bool showAreas = true;
		public bool showSites = true;
		public bool showLists = true;
		public bool showListItems = true;

		protected void Page_Load(object sender, System.EventArgs e)
		{
			showAreas = (ConfigurationSettings.AppSettings["TreeShowArea"] == null) ? showAreas : Convert.ToBoolean(ConfigurationSettings.AppSettings["TreeShowArea"].ToString());
			showSites = (ConfigurationSettings.AppSettings["TreeShowSites"] == null) ? showSites : Convert.ToBoolean(ConfigurationSettings.AppSettings["TreeShowSites"].ToString());
			showLists = (ConfigurationSettings.AppSettings["TreeShowLists"] == null) ? showLists : Convert.ToBoolean(ConfigurationSettings.AppSettings["TreeShowLists"].ToString());
			showListItems = (ConfigurationSettings.AppSettings["TreeShowListItems"] == null) ? showListItems : Convert.ToBoolean(ConfigurationSettings.AppSettings["TreeShowListItems"].ToString());

			action = (Request.Params["action"] != null) ? Request.Params["action"].ToString() : "get";
			//connectedUsername = Request.ServerVariables["LOGON_USER"].ToString();
            connectedUsername = CardioLogUser.GetUserName(CardioLogUser.UserType.LogonUser);
			username = (Request.Params["userName"] != null && !Request.Params["username"].ToString().Equals(string.Empty)) ? Request.Params["username"].ToString() : connectedUsername;
			treeType = (NavigatorType)Convert.ToInt16(Request.Params["treeType"] );
			if ( action == "getTemplates" ) entityType = (Request.Params["entityType"] != null) ? Request.Params["entityType"].ToString() : "Area";

		}

		public string handleAction() 
		{
			switch (action) 
			{
				case "get":
					return getTree();
				case "add":
					return addNode();
				case "addSilent":
					return addNodeSilent();
				case "remove":
					return removeNode();
				case "edit":
					return editNode();
				case "search":
					return searchNodes();
				case "deleteTemplate":
					return deleteTemplate();
				case "publishTemplate":
					return publishTemplate();
				case "createTemplate":
					return createTemplate();
				case "getTemplates":
					return getTemplates();
				case "deleteService":
					return deleteService();
				case "createService":
					return createService();
				case "editService":
					return editService();
				case "newReport":
					return addReport();
				case "createReport":
					return createReport();
				case "createReportFromTemplate":
					return createReportFromTemplate();
				case "removeReport":
					return removeReport();
				case "removeHistoryReport":
					return removeHistoryReport();
				case "editReport":
					return editReport();
				case "copyReport":
					return copyReport();
				case "rename":
					return renameNode();
				case "permissions":
					return setPermissions();
				default:
					return "<status id='-1'>unknown action</status>";
			}
		}
		
		public string searchNodes() 
		{
			// searchTerm - title or url
			// exact - true/false
			// searchOn (optional) - "title" or "url"

			// return id, title, url, displayPath (string name > name > name), path (array ids)

	        StringBuilder result = new StringBuilder("<call>");
	        string searchOn = (Request.Params["searchon"] != null) ? Request.Params["searchOn"].ToString() : "friendlyname";
	        bool exact = (Request.Params["exact"] != null && Request.Params["exact"].ToString() == "true") ? true : false;
	        bool includeDeleted = (Request.Params["deleted"] != null && Request.Params["deleted"].ToString() == "true") ? true : false;
	
	        ArrayList results;
	        try
	        {
	            results = Util.SPManager.Search(Request.Params["term"].ToString(), exact, searchOn, true, includeDeleted);
	        }
	        catch (Exception e)
	        {
	            results = new ArrayList();
	        }
	
	
	        string prefs = "<prefs>" +
	                        "<columns>" +
	                            "<column id='id' title='Id' type='hidden' width='100' />" +
	                            "<column id='path' title='Path' type='hidden' width='100' />" +
	                            "<column id='title' title='Title' type='text' width='200' />" +
	                            "<column id='url' title='Url' width='500' type='text' sorted='true' />" +
	                        "</columns>" +
	                        "</prefs>";
	
	        result.Append("<title>" + CardioLog.API.Localization.getText("$Search Results$")+ "</title>");
	        result.Append(prefs);
	        result.Append("<data><![CDATA[<table>");
	        result.Append(prefs);
	        result.Append("<thead>" +
	                        "<td type='hidden' width='100' id='id'>Id</td>" +
	                        "<td type='hidden' width='100' id='path'>Path</td>" +
	                        "<td type='text' width='200' id='title'>" + CardioLog.API.Localization.getText("$Title$") + "</td>" +
	                        "<td type='text' width='500' id='url'>" + CardioLog.API.Localization.getText("$Url$") + "</td>" +
	                    "</thead>");
	
	        int numberOfResults = (results.Count > 50) ? 50 : results.Count;
	
	        result.Append("<tbody from='0' to='" + numberOfResults + "' total='" + numberOfResults + "'>");
	        for (int i = 0; i < numberOfResults; i++)
	        {
	            result.Append("<tr>" +
	                            "<td>" + ((SharePointItem)results[i]).Id.ToString() + "</td>" +
	                            "<td>" + ((SharePointItem)results[i]).GetIdPath("*") + "</td>" +
	                            "<td>" + Regex.Replace(((SharePointItem)results[i]).FriendlyName, @"&", "&amp;") + "</td>" +
	                            "<td>" + Regex.Replace(((SharePointItem)results[i]).Url, @"&", "&amp;") + "</td>" +
	                            "</tr>");
	        }
	        result.Append("</tbody></table>]]></data></call>");
	
	        return result.ToString();
          
		}

		private string getTemplateName(string prefs) 
		{
			return CardioLog.API.Localization.getText(Regex.Replace(prefs, @"^.*<title>([^<]*)<\/title>.*$", "$1", RegexOptions.IgnoreCase));
		}

		public string getTemplates()
		{
			EntityType type = EntityType.Get(entityType);//(Request.Params["entityType"] != null && !Request.Params["entityType"].Equals(string.Empty)) ? CardioLog.API.EntityType.Get(Request.Params["entityType"].ToString()) : API.EntityType.Null;
			StringBuilder baseTemplates = new StringBuilder();
			DataRow[] rows = API.CLPage.GetAvailableTemplates(type).Select();
			for (int i=0;i<rows.Length;i++) baseTemplates.AppendFormat("createBaseTemplate('"+rows[i]["EntityId"]+"', '"+getTemplateName(rows[i]["Prefs"].ToString())+"');\n");
			return baseTemplates.ToString();
		}

		public string getTemplatesXML(EntityType entityType) 
		{
			StringBuilder result = new StringBuilder();
			DataRow[] rows = API.CLPage.Templates().Select("entityType='"+entityType.Id+"'");
			result.AppendFormat( "<ITEMS id='{0}' total='{1}' children='{1}'>",entityType.ToString(), rows.Length );
			for (int i=0;i<rows.Length;i++) 
			{
				result.AppendFormat("<ITEM>"+
					"<NAME>"+getTemplateName(rows[i]["prefs"].ToString())+"</NAME>"+
					"<URL>template</URL>"+
					"<TYPE>"+entityType.ToString()+"</TYPE>"+
					"<BASETYPE>"+entityType.ToString()+"</BASETYPE>"+
					"<ID>"+rows[i]["EntityId"].ToString()+"</ID>"+
					"<PUBLISH>"+API.CLPage.IsTemplatePublished(new Guid(rows[i]["EntityId"].ToString()), entityType )+"</PUBLISH>"+
					"</ITEM>");

			}
			result.AppendFormat("</ITEMS>");
			return result.ToString();
		}

		public string getServicesXML() 
		{
			StringBuilder result = new StringBuilder();
			HandlersInstances[] services = API.HandlersInstances.All();
			if (services.Length > 0)
			{
				result.AppendFormat("<ITEMS id='1000' total='" + services.Length + "' children='" + services.Length + "'>");
				foreach (HandlersInstances currHandler in services)
				{
					Handlers currService = new Handlers(currHandler.HandlerId);
					result.AppendFormat("<ITEM>"+
						"<NAME>"+currService.HandlerFriendlyName+"</NAME>"+
						"<URL>service</URL>"+
						"<TYPE>service</TYPE>"+
						"<BASETYPE>service</BASETYPE>"+
						"<ID>"+currService.HandlerId+"</ID>"+
						"</ITEM>");
				}
				result.AppendFormat("</ITEMS>");
			}
			else
			{
				result.AppendFormat( "<ITEMS id='1000' total='0' children='0'></ITEMS>" );
			}
			return result.ToString();
		}

		public string deleteService()
		{
			try 
			{
				string serviceId;

				StringBuilder result = new StringBuilder();

				if (Request.Params["serviceId"] == null) throw new Exception("Missing Service Id");
				
				serviceId = Request.Params["serviceId"].ToString();

				try 
				{
					HandlersInstances.Delete(Convert.ToInt32(serviceId)); 

					// delete params
					HandlersParams.DeleteAll(Convert.ToInt32(serviceId));
				}
				catch (Exception e) 
				{
					result.AppendFormat("<status id='-1'>"+e.Message+"</status>");
				}

				result.AppendFormat(this.getServicesXML());
				return result.ToString();
			} 
			catch (Exception e) 
			{
				return "<status id='-1'>"+e.Message+"</status>";
			}
		}
		public string createService() 
		{
			try 
			{
				int serviceId;
				int scheduleType;
				long miliseconds;

				StringBuilder result = new StringBuilder();

				if (Request.Params["serviceId"] == null) throw new Exception("Missing Service Id");
				if (Request.Params["scheduleType"] == null) throw new Exception("Missing Schedule Type");
				if (Request.Params["miliseconds"] == null) throw new Exception("Missing Miliseconds");

				serviceId = Convert.ToInt32(Request.Params["serviceId"]);
				scheduleType = Convert.ToInt32(Request.Params["scheduleType"]);
				miliseconds = Convert.ToInt64(Request.Params["miliseconds"]);

				try 
				{
					HandlersInstances.New(serviceId, scheduleType, miliseconds);
					HandlersParams.New(serviceId, "verbose", "1"); 
				}
				catch (Exception e) 
				{
					result.AppendFormat("<status id='-1'>"+e.Message+"</status>");
				}

				result.AppendFormat(this.getServicesXML());
				return result.ToString();
				
			} 
			catch (Exception e) 
			{
				return "<status id='-1'>"+e.Message+"</status>";
			}
		}
		public string editService() 
		{
			try
			{
				int serviceId;
				int scheduleType;
				long miliseconds;
				string verbose;

				StringBuilder result = new StringBuilder();

				if (Request.Params["serviceId"] == null) throw new Exception("Missing Service Id");
				if (Request.Params["scheduleType"] == null) throw new Exception("Missing Schedule Type");
				if (Request.Params["miliseconds"] == null) throw new Exception("Missing Miliseconds");
				if (Request.Params["verbose"] == null) throw new Exception("Missing Verbose");
			
				serviceId = Convert.ToInt32(Request.Params["serviceId"]);
				scheduleType = Convert.ToInt32(Request.Params["scheduleType"]);
				miliseconds = Convert.ToInt64(Request.Params["miliseconds"]);
				verbose = Request.Params["verbose"].ToString();

				try 
				{
					// Update HandlersInstances
					HandlersInstances service = new HandlersInstances(serviceId);
					service.HandlerId = serviceId;
					service.Miliseconds = miliseconds;
					service.ScheduleType = scheduleType;
					service.Update();
					
					// Update HandlersParams
					HandlersParams pVerbose = new HandlersParams(serviceId, "verbose");
					pVerbose.Val = verbose;
					pVerbose.Update();
				

					result.AppendFormat(this.getServicesXML());
					return result.ToString();
				}
				catch (Exception e) 
				{
					result.AppendFormat("<status id='-1'>"+e.Message+"</status>");
				}

				result.AppendFormat(this.getServicesXML());
				return result.ToString();
					
			} 
			catch (Exception e) 
			{
				return "<status id='-1'>"+e.Message+"</status>";
			}
		}

		public string deleteTemplate() 
		{
			try 
			{
				EntityType entityType;
				string templateId;

				StringBuilder result = new StringBuilder();

				if (Request.Params["entityType"] == null) throw new Exception("Missing Entity Type");
				if (Request.Params["templateId"] == null) throw new Exception("Missing Template Id");

				//entityType = (EntityType)Enum.Parse(typeof(EntityType), Request.Params["entityType"].ToString());
				entityType = EntityType.Get(Request.Params["entityType"].ToString());
				templateId = Request.Params["templateId"].ToString();

				if (templateId.Equals(Guid.Empty.ToString())) throw new Exception("Cannot delete default template"); 
				try 
				{
                    // remove all context-based reports based on this template
                    API.ContextBasedReports.DeleteByTemplateId(new Guid(templateId));

					API.CLPage.DeleteTemplate(new Guid(templateId), entityType);
				}
				catch (Exception e) 
				{
					result.AppendFormat("<status id='-1'>"+e.Message+"</status>");
				}

				result.AppendFormat(this.getTemplatesXML(entityType));
				return result.ToString();
			} 
			catch (Exception e) 
			{
				return "<status id='-1'>"+e.Message+"</status>";
			}
		}

		public string publishTemplate() 
		{
			try 
			{
				EntityType entityType;
				string templateId;

				StringBuilder result = new StringBuilder();

				if (Request.Params["entityType"] == null) throw new Exception("Missing Entity Type");
				if (Request.Params["templateId"] == null) throw new Exception("Missing Template Id");

				entityType = EntityType.Get(Request.Params["entityType"].ToString());
				templateId = Request.Params["templateId"].ToString();

				if (templateId.Equals(Guid.Empty.ToString())) throw new Exception("Cannot publish default template"); 
				try 
				{
					if (API.CLPage.IsTemplatePublished(new Guid(templateId), entityType))
						API.CLPage.UnPublishTemplate(new Guid(templateId), entityType);
					else
						API.CLPage.PublishTemplate(new Guid(templateId), entityType);
				}
				catch (Exception e) 
				{
					result.AppendFormat("<status id='-1'>"+e.Message+"</status>");
				}

				result.AppendFormat(this.getTemplatesXML(entityType));
				return result.ToString();
			} 
			catch (Exception e) 
			{
				return "<status id='-1'>"+e.Message+"</status>";
			}
		}

		public string createTemplate() 
		{
			try 
			{
				EntityType entityType;
				string templateName;
				string baseTemplateId;
				int columns;

				StringBuilder result = new StringBuilder();

				if (Request.Params["entityType"] == null) throw new Exception("Missing Entity Type");
				if (Request.Params["templateName"] == null) throw new Exception("Missing Template Name");

				//entityType = (EntityType)Enum.Parse(typeof(EntityType), Request.Params["entityType"].ToString());
				entityType = EntityType.Get(Request.Params["entityType"].ToString());
				templateName = Request.Params["templateName"].ToString();
				baseTemplateId = Request.Params["baseTemplateId"].ToString();
				columns = (Request.Params["numOfColumns"] == null) ? 2 : Convert.ToInt32(Request.Params["numOfColumns"].ToString());

				try 
				{
					API.CLPage.CreateTemplate(entityType, templateName, baseTemplateId, columns);
				}
				catch (Exception e) 
				{
					result.AppendFormat("<status id='-1'>"+e.Message+"</status>");
				}

				result.AppendFormat(this.getTemplatesXML(entityType));
				return result.ToString();
			} 
			catch (Exception e) 
			{
				return "<status id='-1'>"+e.Message+"</status>";
			}
		}

		public string removeNode() 
		{
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username, connectedUsername );

			try 
			{
				if (Request.Params["entityId"] == null) throw new Exception("Missing Entity Id");
				string Id = Request.Params["entityId"].ToString();

				SharePointItem item = vtManager[new Guid(Id)];

				if (!item.DeletePermission) throw new Exception(CardioLog.API.Localization.getText("$You are not authorized$") + ".");

				SharePointItem parent;
				if (item.Id == vtManager.Root.Id) 
				{
					parent = vtManager.Root;
					SharePointItem[] childs = item.ChildrenByField( (IComparer)new NameComparer() );
					foreach (SharePointItem child in childs ) 
					{
						SharePointItem currChild = vtManager[child.Id];
						vtManager.Delete(currChild);
					}
				} 
				else 
				{
					parent = vtManager[item.ParentId];
					vtManager.Delete(item);
				}

				result.AppendFormat(getTree(parent.Id.ToString()));
			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}

			return result.ToString();
		}

		public string editNode() 
		{
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username, connectedUsername );

			try 
			{
				if (Request.Params["entityUrl"] == null) throw new Exception("Missing Entity Url");
				if (Request.Params["entityName"] == null) throw new Exception("Missing Entity Name");
				if (Request.Params["id"] == null) throw new Exception("Missing Entity Id");

				string url = Request.Params["entityUrl"].ToString();
				string name = Request.Params["entityName"].ToString();
				string id = Request.Params["id"].ToString();

				SharePointItem node = vtManager[new Guid(id)];

				if (!node.UpdatePermission) throw new Exception(CardioLog.API.Localization.getText("$You are not authorized$") + ".");

				node.Name =  name;
				node.Url = url;
				
				vtManager.Update(node);

				SharePointItem parent = vtManager[node.ParentId];

				result.AppendFormat(getTree(parent.Id.ToString(), id));
			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}

			return result.ToString();
		}

		public string addNodeSilent() 
		{
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username );
			
			try 
			{
				if (Request.Params["entityId"] == null) throw new Exception("Missing Id");

				Guid entityId = new Guid(Request.Params["entityId"].ToString());

				string command = (Request.Params["command"] != null) ? Request.Params["command"].ToString() : string.Empty;
				string controlId = (Request.Params["controlId"] != null) ? Request.Params["controlId"].ToString() : string.Empty;

				SharePointItem item = Util.SPManager.Object(entityId);

				SharePointItem parent = vtManager.Root;

				// check if entity exists already in managed tree
				SharePointItem node = vtManager[entityId];
				if (node != null) throw new Exception(CardioLog.API.Localization.getText("$One instance error$"));
				
				node = vtManager.New(entityId, item.Type, item.Url, item.FriendlyName, parent);

				Guid templateId = (Request.Params["templateId"] != null) ? new Guid(Request.Params["templateId"].ToString()) : Guid.Empty;

				if (templateId != Guid.Empty) 
				{
					API.CLPage page = new API.CLPage(node.Id, username, node.Type, templateId);
					page.RestoreToDefault(templateId);
				}

				// Refresh managed tree
				vtManager = Util.Refresh( username );

				if (!command.Equals(string.Empty)) 
				{
					switch (command) 
					{
						case "remove":
							//API.CLPage page = new API.CLPage(entityId, username, item.Type);
							//page.RemoveControl(Convert.ToInt32(controlId));
							break;
						case "hide":
							break;
						default:
							break;
					}
				}
				result.AppendFormat(getTree(parent.Id.ToString(), entityId.ToString()));
			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}

			return result.ToString();
		}

		public string addNode() 
		{
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username, connectedUsername );
			
			try 
			{
				if (Request.Params["entityType"] == null) throw new Exception("Missing Entity Type");
				if (Request.Params["entityUrl"] == null) throw new Exception("Missing Entity Url");
				if (Request.Params["entityName"] == null) throw new Exception("Missing Entity Name");
				if (Request.Params["entityParentId"] == null) throw new Exception("Missing Parent");

				//EntityType entityType = (EntityType)Enum.Parse(typeof(EntityType), Request.Params["entityType"].ToString());
				EntityType entityType;
				if (Regex.Match(Request.Params["entityType"].ToString(), "[a-z]", RegexOptions.IgnoreCase).Success) 
					entityType = EntityType.Get(Request.Params["entityType"].ToString());
				else
					entityType = EntityType.Get(Convert.ToInt16(Request.Params["entityType"].ToString()));

				string url = Request.Params["entityUrl"].ToString();
				string name = Request.Params["entityName"].ToString();
				string parentId = Request.Params["entityParentId"].ToString();

				Guid entityId = Guid.Empty;

				if ( entityType == EntityType.Area ||
					entityType == EntityType.List ||
					entityType == EntityType.ListItem ||
					entityType == EntityType.Site )
					entityId = Util.SPManager[url].Id;
				else
					entityId = Guid.NewGuid();

				// check if entity exists already in managed tree
				SharePointItem node = vtManager[entityId];
				if (node != null) throw new Exception(CardioLog.API.Localization.getText("$One instance error$"));

				SharePointItem parent = vtManager[new Guid(parentId)];
                if (!Demo.IsDemo)
                {
					if (!parent.InsertPermission) throw new Exception(CardioLog.API.Localization.getText("$You are not authorized$") + ".");
                }

				node = vtManager.New(entityId, entityType, url, name, parent);

                // Set initial permissions for a folder in the report center
                if (entityType == EntityType.Folder && username == "CARDIOLOG_REPORTS")
                {
                    setInitialPermissions(connectedUsername, entityId);
                }

				// Refresh managed tree
				vtManager = Util.Refresh( connectedUsername );

				result.AppendFormat(getTree(parent.Id.ToString(), entityId.ToString()));
			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}

			return result.ToString();
		}

		public string getTree() 
		{
			StringBuilder result = new StringBuilder();
			string[] ids = Request["id"].Split(new char[] { ','} );
			foreach (string id in ids ) result.Append(getTree(id));
			return result.ToString();
		}

		public string getTree(string id) 
		{
			return getTree(id, string.Empty);
		}

		public string getTree(string id, string childId) 
		{
			try 
			{
				StringBuilder result = new StringBuilder();
		
				username = Regex.Replace(username, @"\\\\", "\\");
			
				VTManager vtManager = Util.VTManager( username, connectedUsername );

				SharePointItem item = null;
			
				switch( treeType )
				{
					case NavigatorType.Physical:
						item = Util.SPManager.Object( new Guid(id) );
						break;
					case NavigatorType.Virtual:
						item = vtManager[new Guid(id)];
						break;
				}
                SharePointItem[] childs;
                string getTreeChildItemsInOneGo = Util.GetWebConfigAppSettingsValue("GetTreeChildItemsInOneGo").ToLower();
                if (getTreeChildItemsInOneGo == "true" && treeType == NavigatorType.Virtual)
                {
                    childs = item.ChildrenByFieldVirtualSQL(new TypeNameComparer());
                }
                else
                {
                    childs = item.ChildrenByField(new TypeNameComparer());
                }

				int childrenCount = 0;
				int totalchildrenCount = 0;
			
				StringBuilder childrenResult = new StringBuilder();

				foreach( SharePointItem child in childs )
				{
					// TODO: handle dynamic entity types
					if ((child.Type == EntityType.Area && !showAreas) ||
						(child.Type == EntityType.Site && !showSites) ||
						(child.Type == EntityType.List && !showLists) ||
						(child.Type == EntityType.ListItem && !showListItems) || child.Hidden) continue;
				
					if (!child.IsDeleted) childrenCount += 1;
					totalchildrenCount += 1;

					childrenResult.Append( "<ITEM deleted='"+child.IsDeleted+"' "+((child.Id.ToString().Equals(childId)) ? "selected='true'" : "")+">" );

					childrenResult.AppendFormat( "<PATH><![CDATA[{0}]]></PATH>", child.DisplayPath(Util.SPManager.Root));
					childrenResult.AppendFormat( "<ID>{0}</ID>",child.Id );
					childrenResult.AppendFormat( "<NAME><![CDATA[{0}]]></NAME>",child.FriendlyName );
					childrenResult.AppendFormat( "<URL><![CDATA[{0}]]></URL>",child.Url );

					string type = string.Empty;
					if (child.Type == null) throw new Exception("Unrecognized node type");

					if (child.FriendlyName.Equals("Home")) type = "Home";
					else if (child.Type.ToString().Equals("List")) type = child.Template;
					else type = child.Type.ToString();

					childrenResult.AppendFormat( "<TYPE>{0}</TYPE>",type );
					childrenResult.AppendFormat( "<BASETYPE>{0}</BASETYPE>",child.Type );

					childrenResult.AppendFormat( "<PERMISSION insert='{0}' update='{1}' delete='{2}'/>", child.InsertPermission, child.UpdatePermission, child.DeletePermission);

                    if (getTreeChildItemsInOneGo == "true")
                    {
                        childrenResult.AppendFormat("<ITEMS id='{0}' children='{1}' total='{2}'></ITEMS>", child.Id, child.SubItemCount, -1);
                    }
                    else
                    {
                        ArrayList children = child.Children;

                        if (children.Count > 0)
                        {
                            int gchildrenCount = 0;
                            int totalgchildrenCount = 0;

                            children.Sort((treeType == NavigatorType.Physical) ? (IComparer)new TemplateComparer() : (IComparer)new NameComparer());

                            //						SharePointItem[] gchilds = new SharePointItem[children.Count];
                            //						children.CopyTo(gchilds);

                            foreach (SharePointItem gchild in children)
                            {
                                if ((gchild.Type == EntityType.Area && !showAreas) ||
                                    (gchild.Type == EntityType.Site && !showSites) ||
                                    (gchild.Type == EntityType.List && !showLists) ||
                                    (gchild.Type == EntityType.ListItem && !showListItems) || gchild.Hidden) continue;
                                if (!gchild.IsDeleted) gchildrenCount += 1;
                                totalgchildrenCount += 1;
                            }
                            childrenResult.AppendFormat("<ITEMS id='{0}' children='{1}' total='{2}'></ITEMS>", child.Id, gchildrenCount, totalgchildrenCount);
                        }
                    }

					childrenResult.Append( "</ITEM>" );
				}
				result.AppendFormat( "<ITEMS id='{0}' children='{1}' total='{2}' treeType='{3}'>", id, childrenCount, totalchildrenCount, (int)treeType);
				result.Append(childrenResult.ToString());
				result.Append( "</ITEMS>" );

				return result.ToString();
			} 
			catch (Exception e) 
			{
				return "<ITEMS id='"+id+"' children='1'>"+
							"<ITEM>"+
								"<PATH></PATH>"+
								"<ID>"+Guid.NewGuid()+"</ID>"+
								"<NAME><![CDATA["+e.Message+"]]></NAME>"+
								"<URL></URL>"+
								"<TYPE>Error</TYPE>"+
								"<BASETYPE>Error</BASETYPE>"+
							"</ITEM>"+
						"</ITEMS>";
			}
		}

		public string addReport() 
		{
			// create report from skretch
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username, connectedUsername );
			
			try 
			{
				if (Request.Params["entityName"] == null) throw new Exception("Missing Entity Name");
				if (Request.Params["entityParentId"] == null) throw new Exception("Missing Parent");

				string description = (Request.Params["entityDescription"] != null) ? Request.Params["entityDescription"].ToString() : " ";
				string name = Request.Params["entityName"].ToString();
				string parentId = Request.Params["entityParentId"].ToString();
				int numberOfColumns = (Request.Params["numOfColumns"] != null) ? Convert.ToInt32(Request.Params["numOfColumns"].ToString()) : 2;
				int scheduleType = (Request.Params["scheduleType"] != null) ? Convert.ToInt32(Request.Params["scheduleType"].ToString()) : 0;

				Guid entityId = Guid.Empty;
				entityId = Guid.NewGuid();

				SharePointItem parent = vtManager[new Guid(parentId)];
				VirtualTreePermissions permission = new VirtualTreePermissions(username, connectedUsername, parent.Id);
				if (!permission.AllowInsert) throw new Exception(CardioLog.API.Localization.getText("$You are not authorized$") + ".");

				SharePointItem node = vtManager.NewReport(entityId, description, name, numberOfColumns, scheduleType, parent);

                setInitialPermissions(connectedUsername, node.Id);

				result.AppendFormat(getTree(parent.Id.ToString(), entityId.ToString()));
			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}

			return result.ToString();
		}

		public string createReportFromTemplate()
		{
			// create report from template based on managed item
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username, connectedUsername );
			
			try 
			{
				if (Request.Params["entityName"] == null) throw new Exception("Missing Entity Name");
				if (Request.Params["entityParentId"] == null) throw new Exception("Missing Parent");
				if (Request.Params["basedOnId"] == null) throw new Exception("Missing item id to base this report on");
				if (Request.Params["basedOnTemplateId"] == null) throw new Exception("Missing template to base this report on");

                SharePointItem basedOnItem;
                if (CardioLog.API.CardioLogUIVersion.UIVersion == "2010")
                {
                    //now base on root 
                    string basedOnId = Util.SPManager.Root.Id.ToString();
                    basedOnItem = Util.SPManager.Object(new Guid(basedOnId));
                    //get entity type
                    string basedOnEntityType = Request.Params["basedOnEntityType"].ToString();
                    basedOnItem.Type = EntityType.Get(basedOnEntityType);
                }
                else
                {
                    string basedOnId = Request.Params["basedOnId"].ToString();
                    basedOnItem = Util.SPManager.Object(new Guid(basedOnId));
                }
				
				//string basedOnId = Request.Params["basedOnId"].ToString();
				//SharePointItem basedOnItem = Util.SPManager.Object(new Guid(basedOnId));
				if (basedOnItem == null) throw new Exception("Could not find based on item");

				string templateId = Request.Params["basedOnTemplateId"].ToString();
				
				string description = (Request.Params["entityDescription"] != null) ? Request.Params["entityDescription"].ToString() : " ";
				string name = Request.Params["entityName"].ToString();
				string parentId = Request.Params["entityParentId"].ToString();
				int scheduleType = (Request.Params["scheduleType"] != null) ? Convert.ToInt32(Request.Params["scheduleType"].ToString()) : 0;

				SharePointItem parent = vtManager[new Guid(parentId)];
				VirtualTreePermissions permission = new VirtualTreePermissions(username, connectedUsername, parent.Id);
				if (!permission.AllowInsert) throw new Exception(CardioLog.API.Localization.getText("$You are not authorized$") + ".");
				
				// Create report from template
				SharePointItem node = vtManager.CopyTemplate("DEFAULT", "CARDIOLOG_REPORTS", new Guid(templateId), description, name, scheduleType, basedOnItem, string.Empty, string.Empty, parent, vtManager);

                setInitialPermissions(connectedUsername, node.Id);

				result.AppendFormat(getTree(parent.Id.ToString(), node.Id.ToString()));
			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}
			return result.ToString();
		}

		public string createReport() 
		{
			// create report based on managed item
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username, connectedUsername );
			
			try 
			{
				if (Request.Params["entityName"] == null) throw new Exception("Missing Entity Name");
				if (Request.Params["entityParentId"] == null) throw new Exception("Missing Parent");
				if (Request.Params["basedOnId"] == null) throw new Exception("Missing item to base this report on");

				string basedOnId = Request.Params["basedOnId"].ToString();

				VTManager managedTree = Util.VTManager( connectedUsername );
				SharePointItem basedOnItem = managedTree[new Guid(basedOnId)];
				if (basedOnItem == null) throw new Exception("Could not find item in managed tree");

				string description = (Request.Params["entityDescription"] != null) ? Request.Params["entityDescription"].ToString() : " ";
				string name = Request.Params["entityName"].ToString();
				string parentId = Request.Params["entityParentId"].ToString();
				int scheduleType = (Request.Params["scheduleType"] != null) ? Convert.ToInt32(Request.Params["scheduleType"].ToString()) : 0;

				Guid entityId = Guid.Empty;
				entityId = Guid.NewGuid();

				SharePointItem parent = vtManager[new Guid(parentId)];
				VirtualTreePermissions permission = new VirtualTreePermissions(username, connectedUsername, parent.Id);
				if (!permission.AllowInsert) throw new Exception(CardioLog.API.Localization.getText("$You are not authorized$") + ".");

				SharePointItem node = vtManager.CopyItem(basedOnItem, connectedUsername, description, name, scheduleType, parent, vtManager);

                setInitialPermissions(connectedUsername, node.Id);

				result.AppendFormat(getTree(parent.Id.ToString(), node.Id.ToString()));
			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}
			return result.ToString();
		}
		
		public string editReport() 
		{
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username );
			try 
			{
				if (Request.Params["entityId"] == null) throw new Exception("Missing Entity Id");
				if (Request.Params["entityName"] == null) throw new Exception("Missing Entity Name");

				string description = (Request.Params["entityDescription"] != null) ? Request.Params["entityDescription"].ToString() : " ";
				string name = Request.Params["entityName"].ToString();
				string entityId = Request.Params["entityId"].ToString();
				int numberOfColumns = (Request.Params["numOfColumns"] != null) ? Convert.ToInt32(Request.Params["numOfColumns"].ToString()) : 2;
				int scheduleType = (Request.Params["scheduleType"] != null) ? Convert.ToInt32(Request.Params["scheduleType"].ToString()) : 0;

				SharePointItem node = vtManager.EditReport(new Guid(entityId), description, name, numberOfColumns, scheduleType);
				result.AppendFormat(getTree(node.ParentId.ToString(), entityId.ToString()));
			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}
			return result.ToString();
		}

		public string copyReport() 
		{
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username, connectedUsername );
			
			try 
			{
				if (Request.Params["reportId"] == null) throw new Exception("Missing report id to base this report on");
				if (Request.Params["entityId"] == null) throw new Exception("Missing Entity Id");
				if (Request.Params["entityName"] == null) throw new Exception("Missing Entity Name");
				if (Request.Params["location"] == null) throw new Exception("Missing Location");

				string parentId = Request.Params["location"].ToString();
				string reportId = Request.Params["reportId"].ToString();

				SharePointItem parentItem = vtManager[new Guid(parentId)];
				if (parentItem == null) throw new Exception("Could not find parent in managed tree");
				SharePointItem currentItem = vtManager[new Guid(reportId)];
				if (currentItem == null) throw new Exception("Could not find current report in managed tree");

				// Get details from current report
				string description = currentItem.Url; // report description 
                API.CLReport currReport = new CardioLog.API.CLReport(new Guid(reportId), "CARDIOLOG_REPORTS");
				int scheduleType = currReport.GetScheduleType(); // report schedule type
				API.CLPage currPage = new CardioLog.API.CLPage(new Guid(reportId), "CARDIOLOG_REPORTS", EntityType.Report);
				int numOfColumns = currPage.NumberOfColumns;
				
				VirtualTreePermissions permission = new VirtualTreePermissions(username, connectedUsername, parentItem.Id);
				if (!permission.AllowInsert) throw new Exception(CardioLog.API.Localization.getText("$You are not authorized$") + ".");

				string[] entityIds = Request.Params["entityId"].Split(',');
				string[] entityNames = Request.Params["entityName"].Split(',');

                string name = string.Empty;
                if (CardioLog.API.CardioLogUIVersion.UIVersion == "2007")
                {
                    name = currentItem.Name;
                    for (int i = 0; i < entityIds.Length; i++)
                    {
                        SharePointItem node =
                            vtManager.CopyReport(reportId, name + " - " + entityNames[i], description, numOfColumns, scheduleType, entityNames[i], entityIds[i], parentItem, vtManager);
                        setInitialPermissions(connectedUsername, node.Id);
                    }
                }

                if (CardioLog.API.CardioLogUIVersion.UIVersion == "2010")
                {
                    name = Request.Params["newreportName"];
                    for (int i = 0; i < entityIds.Length; i++)
                    {
                        SharePointItem node = 
                            vtManager.CopyReport(reportId, name, description, numOfColumns, scheduleType, entityNames[i], entityIds[i], parentItem, vtManager);

                        setInitialPermissions(connectedUsername, node.Id);
                    }
                }

				//  Refresh Tree
				result.AppendFormat(getTree(parentItem.Id.ToString()));
			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}
			return result.ToString();
		}

		private SharePointItem removeReport(VTManager vtManager, Guid entityId) 
		{
			// remove clpage and all historical versions
			SharePointItem item = vtManager[entityId];
			SharePointItem parent = vtManager[item.ParentId];

			// remove all reports data (including history data)
			CardioLog.API.CLReport.Delete(entityId, username);

			vtManager.Delete(item);

            // remove all context-based reports based on this report
            CardioLog.API.ContextBasedReports.DeleteByTemplateId(entityId);

			return parent;
		}

		public string removeHistoryReport() 
		{
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username, connectedUsername );
			try 
			{
				if (Request.Params["entityId"] == null) throw new Exception("Missing Id");

				string Id = Request.Params["entityId"].ToString();

				VirtualTreePermissions permission = new VirtualTreePermissions(username, connectedUsername, new Guid(Id));
				if (!permission.AllowDelete) throw new Exception(CardioLog.API.Localization.getText("$You are not authorized$") + ".");

				CardioLog.API.CLReport.DeleteHistory(new Guid(Id), username);
				result.AppendFormat(getTree(Id));

			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}

			return result.ToString();
		}

		public string removeReport() 
		{
			StringBuilder result = new StringBuilder();
			VTManager vtManager = Util.VTManager( username, connectedUsername );
			try 
			{
				if (Request.Params["entityId"] == null) throw new Exception("Missing Id");

				string Id = Request.Params["entityId"].ToString();

				VirtualTreePermissions permission = new VirtualTreePermissions(username, connectedUsername, new Guid(Id));
				if (!permission.AllowDelete) throw new Exception(CardioLog.API.Localization.getText("$You are not authorized$") + ".");

				result.AppendFormat(getTree(removeReport(vtManager, new Guid(Id)).Id.ToString()));

			} 
			catch (Exception e) 
			{
				result.AppendFormat("<STATUS id='-1'>"+e.Message+"</STATUS>");
			}

			return result.ToString();
		}

        private void setInitialPermissions(string connectedUserName, Guid entityId)
        {
            try
            {
                // Set all permissions for the creator
                bool _delete = true;
                bool _view = true;
                bool _update = true;
                
                if (Demo.IsDemo)
                {
                    _delete = false;
                    _view = false;
                    _update = false;
                }

                VirtualTreePermissions permissionRow = VirtualTreePermissions.New("CARDIOLOG_REPORTS", connectedUserName, entityId, true, _update, true, _delete, false, false);
                
                VirtualTreePermissions allGroup = VirtualTreePermissions.New("CARDIOLOG_REPORTS", string.Empty, entityId, _view, false, false, false, true, false);
                
            }
            catch(Exception ex)
            {
                throw new ApplicationException("Could not set initial permissions for the report: " + ex.Message );
            }
        }

		public string renameNode()
		{
			StringBuilder result = new StringBuilder();

			try
			{
				if (Request.Params["id"] == null) throw new Exception("Missing node id");
				if (Request.Params["name"] == null) throw new Exception("Missing node name");

				string id = Request.Params["id"].ToString();
				string name = Request.Params["name"].ToString();

				SharePointItem node = Util.SPManager.Object( new Guid(id) );

				node.Rename(name);

				// Clear cached tree
				TreeWS treeWS = new TreeWS();
				treeWS.Invalidate();

				result.AppendFormat(getTree(node.ParentId.ToString()));
			}
			catch(Exception ex)
			{
				result.AppendFormat("<STATUS id='-1'>"+ex.Message+"</STATUS>");
			}
			return result.ToString();
		}

		public string setPermissions()
		{
			StringBuilder result = new StringBuilder();

			try
			{
				SharePointItem root = Util.SPManager.Root;

				// Clear cached tree per user
				TreeWS treeWS = new TreeWS();
				treeWS.InvalidateUser(connectedUsername);

				// Refresh virtual tree
				VTManager vtManager = Util.Refresh(connectedUsername);

				result.AppendFormat(getTree(root.Id.ToString()));
			}
			catch(Exception ex)
			{
				result.AppendFormat("<STATUS id='-1'>"+ex.Message+"</STATUS>");
			}
			return result.ToString();
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
		}
		#endregion
	}
}
